"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AssetsPlugin = require("assets-webpack-plugin");
const chalk_1 = require("chalk");
const CleanWebpackPlugin = require("clean-webpack-plugin");
const fs = require("fs");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const lodash_1 = require("lodash");
const path = require("path");
const TerserPlugin = require("terser-webpack-plugin");
const webpack = require("webpack");
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const WorkboxPlugin = require("workbox-webpack-plugin");
const get_bundles_info_1 = require("../../components/project_deps/get_bundles_info");
const env_1 = require("../../config/env");
const pkg_1 = require("../../config/pkg");
const rearguard_1 = require("../../config/rearguard");
const const_1 = require("../../const");
const helpers_1 = require("../../helpers");
const index_1 = require("./../../config/analyze/index");
// tslint:disable:variable-name
exports.DllPlugin = () => {
    return [
        ...exports.clean([helpers_1.dll_output_path()], true),
        new webpack.DllPlugin({
            context: helpers_1.get_context(),
            name: helpers_1.dll_entry_name(),
            path: helpers_1.dll_manifest_path(),
        }),
        new webpack.optimize.OccurrenceOrderPlugin(false),
    ];
};
exports.DllReferencePlugin = (exclude_them_self = false) => {
    const bundlesInfo = get_bundles_info_1.get_bundles_info();
    const plugins = [];
    for (const { has_dll, bundle_entry_name, manifest, load_on_demand } of bundlesInfo) {
        if (!load_on_demand && has_dll) {
            if (path.isAbsolute(manifest) && fs.existsSync(manifest)) {
                plugins.push(new webpack.DllReferencePlugin({
                    context: helpers_1.get_context(),
                    manifest,
                    name: bundle_entry_name.dll,
                }));
            }
            else {
                console.log(chalk_1.default.red(`[ DllReferencePlugin ][ ERROR ][ MANIFEST_NOT_FOUND: ${manifest} ]`));
                process.exit(1);
            }
        }
    }
    // exclude_them_self - выставляется в true в случае когда dll зависит от dll.
    if (!exclude_them_self) {
        if (fs.existsSync(helpers_1.dll_manifest_path())) {
            plugins.push(new webpack.DllReferencePlugin({
                context: helpers_1.get_context(),
                manifest: helpers_1.dll_manifest_path(),
                name: helpers_1.dll_entry_name(),
            }));
        }
    }
    return plugins;
};
class ComputeDataForHWP {
    apply(compiler) {
        compiler.hooks.compilation.tap("compute_data_for_html_webpack_plugin", (compilation) => {
            HtmlWebpackPlugin.getHooks(compilation).beforeAssetTagGeneration.tapAsync("compute_data_for_html_webpack_plugin", // <-- Set a meaningful name here for stacktraces
            (compilation_data, callback) => {
                const bundlesInfo = get_bundles_info_1.get_bundles_info();
                const data = { js: [], css: [] };
                for (const { assets, bundle_name, has_dll, has_ui_lib, load_on_demand } of bundlesInfo) {
                    if (!load_on_demand) {
                        if (has_dll && fs.existsSync(assets.dll)) {
                            data.js.push(require(assets.dll)[helpers_1.dll_entry_name(bundle_name)].js);
                        }
                        if (has_ui_lib && fs.existsSync(assets.lib)) {
                            data.js.push(require(assets.lib)[helpers_1.lib_entry_name(bundle_name)].js);
                        }
                    }
                }
                if (!rearguard_1.rearguardConfig.load_on_demand && fs.existsSync(helpers_1.dll_assets_path())) {
                    data.js.push(require(helpers_1.dll_assets_path())[helpers_1.dll_entry_name()].js);
                }
                // Manipulate the content
                compilation_data.assets.js = [...data.js, ...compilation_data.assets.js];
                console.log("");
                console.log(chalk_1.default.bold.green("[ HtmlWebpackPlugin ][ BUILD ][ index.html ]"));
                for (const css of compilation_data.assets.css) {
                    console.log(chalk_1.default.green(`[ CSS ][ ${css} ]`));
                }
                for (const js of compilation_data.assets.js) {
                    console.log(chalk_1.default.green(`[ JS ][ ${js} ]`));
                }
                console.log("");
                // Tell webpack to move on
                callback(null, compilation_data);
            });
        });
    }
}
exports.htmlWebpackPlugin = () => {
    const { isWDS, isBuild } = env_1.envConfig;
    if (isWDS || isBuild) {
        return [
            new HtmlWebpackPlugin({
                filename: "index.html",
                inject: false,
                template: path.resolve(__dirname, "../../../../templates/html-webpack-template.ejs"),
                // tslint:disable-next-line:object-literal-sort-keys
                meta: { viewport: "width=device-width, initial-scale=1, shrink-to-fit=no" },
            }),
            new ComputeDataForHWP(),
        ];
    }
    return [];
};
exports.HMR = () => {
    const { isDevelopment, isBuild } = env_1.envConfig;
    if (isDevelopment && !isBuild) {
        return [
            // prints more readable module names in the browser console on HMR updates
            new webpack.NamedModulesPlugin(),
            // enable HMR globally
            new webpack.HotModuleReplacementPlugin(),
        ];
    }
    return [];
};
exports.uglify = () => {
    const { isDevelopment } = env_1.envConfig;
    if (!isDevelopment) {
        return [
            // https://github.com/webpack-contrib/terser-webpack-plugin
            new TerserPlugin({
                cache: true,
                parallel: true,
                terserOptions: {
                    compress: {
                        sequences: true,
                        unused: true,
                    },
                    ecma: 8,
                },
            }),
        ];
    }
    return [];
};
exports.workboxPlugin = () => {
    const { isDevelopment } = env_1.envConfig;
    if (!isDevelopment) {
        return [
            new WorkboxPlugin.GenerateSW({
                clientsClaim: true,
                globDirectory: helpers_1.get_output_path(),
                globPatterns: ["*.{js,html,jpeg,jpg,svg,gif,png,ttf}"],
                importWorkboxFrom: "local",
                maximumFileSizeToCacheInBytes: 10 * 1024 * 1024,
                navigateFallback: "/",
                skipWaiting: true,
                swDest: "sw.js",
            }),
        ];
    }
    return [];
};
// Webpack Bundle Analyzer
// https://github.com/th0r/webpack-bundle-analyzer
exports.analyze = () => {
    const { isDebug } = env_1.envConfig;
    const { port } = index_1.analyzeConfig;
    if (isDebug) {
        return [
            new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                analyzerPort: port,
            }),
        ];
    }
    return [];
};
exports.clean = (toRemove = [], force = false) => {
    const { isDevelopment, isBuild, isDebug } = env_1.envConfig;
    if (!isDevelopment || force || isBuild) {
        return [new CleanWebpackPlugin(toRemove, { root: process.cwd(), verbose: isDebug })];
    }
    return [];
};
// tslint:disable:variable-name
exports.assetsPlugin = (bundle_dir) => {
    return [
        new AssetsPlugin({
            path: path.resolve(env_1.envConfig.rootDir, bundle_dir, lodash_1.snakeCase(pkg_1.pkgInfo.name), const_1.BUNDLE_SUB_DIR()),
            // tslint:disable-next-line:object-literal-sort-keys
            filename: const_1.ASSETS_NAME,
            processOutput(data) {
                const result = {};
                for (const key in data) {
                    if (key.indexOf(lodash_1.snakeCase(pkg_1.pkgInfo.name)) !== -1) {
                        result[key] = data[key];
                    }
                }
                return JSON.stringify(result, null, 2);
            },
        }),
    ];
};
// tslint:enable:variable-name
